Zetor7340Turbo = {};

function Zetor7340Turbo.prerequisitesPresent(specializations)
	return true;
end;

function Zetor7340Turbo:load(xmlFile)

	self.exhaustFlap = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.exhaustFlap#index"));
	
	self.backupMinRpm = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#minRpm"), 1000);
	local motorMaxRpmStr = getXMLString(xmlFile, "vehicle.motor#maxRpm");
    local motorMaxRpm1, motorMaxRpm2, motorMaxRpm3 = Utils.getVectorFromString(motorMaxRpmStr);
    motorMaxRpm1 = Utils.getNoNil(motorMaxRpm1, 800);
    motorMaxRpm2 = Utils.getNoNil(motorMaxRpm2, 1000);
    motorMaxRpm3 = Utils.getNoNil(motorMaxRpm3, 1800);
	self.fieldMode = true;
	self.backUpBrakeForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#brakeForce"), 10)*2;
	self.backUpBackwardGearRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#backwardGearRatio"), 1.5);
	self.backUpMaxRpm = motorMaxRpm3;
	self.temp = 25;
	self.coolingOn = false;
	self.coolingTemp = 0;
	self.tempOnBord = true;
	
	
	local numWorkLights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.worklights#count"), 0);
    self.worklights = {};
    for i=1, numWorkLights do
        local worklightnamei = string.format("vehicle.worklights.worklight%d", i);
        self.worklights[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, worklightnamei .. "#index"));
        setVisibility(self.worklights[i], false);
    end;
	self.worklightsActive = false;
	self.fourWheelMode = false;
	self.warmUp = 0.025;
	self.isStarted = false;
	
	self.helpPanelPath = Utils.getFilename("textures/Zetor7340TurboHUD.png", self.baseDirectory);
	self.HudWidth = 0.386;
	self.HudHeight = 0.252; --old 0.256;
	self.HudPosX = 0.001;
	self.HudPosY = 0.001;
	self.helpPanelTextPosX = 0.002;
	self.helpPanelTextPosY = 0.356;
	self.helpPanelOverlay = Overlay:new("Helppanel", self.helpPanelPath, self.HudPosX, self.HudPoxY, self.HudWidth, self.HudHeight);
	self.helpPanelActive = false;

    -- Revers lights
    self.reverseLightObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.reverse#index"));
    setVisibility(self.reverseLightObject, false);
    self.reverseLightActive = false;
    
    -- Brake lights
    self.brakeLightObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.brake#index"));
    setVisibility(self.brakeLightObject, false);
    self.brakeLightActive = false;
    
    -- Turn lights
    self.leftTurnObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.leftTurn#index"))
    setVisibility(self.leftTurnObject, false);
    self.leftTurnActive = false;
    self.flashLeft = false;
    
    self.rightTurnObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.rightTurn#index"))
    setVisibility(self.rightTurnObject, false);
    self.rightTurnActive = false;
    self.flashRight = false;
    
    self.flash = false;
    self.flashWarn = false;
    self.timer = 10;
end;

function Zetor7340Turbo:delete()
end;

function Zetor7340Turbo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Zetor7340Turbo:keyEvent(unicode, sym, modifier, isDown)

    -- Turn signals
    if InputBinding.isPressed(InputBinding.LEFTTURN) then
        if not self.flashLeft or self.flashWarn then
            self.flashLeft = false;
        end;
        self.flashLeft = not self.flashLeft;
        self.flashRight = false;
        self.flashWarn = false;
        self.timer = 10;
    end;
    if InputBinding.isPressed(InputBinding.RIGHTTURN) then
        if not self.flashRight or self.flashWarn then
            self.flashRight = false;
        end;
        self.flashRight = not self.flashRight;
        self.flashLeft = false;
        self.flashWarn = false;
        self.timer = 10;
    end;
    -- Warn lights
    if InputBinding.isPressed(InputBinding.WARNLIGHTS) then
        if self.flashWarn then
            self.flashLeft = false;
            self.flashRight = false;
        else
            self.rightTurnActive = false;
            self.leftTurnActive = false;
            self.flashLeft = true;
            self.flashRight = true;
        end;
        self.flashWarn = not self.flashWarn;
        self.timer = 10;
    end;
end;

function Zetor7340Turbo:stopMotor()
    self.motor.minRpm = self.backupMinRpm;
end;

function Zetor7340Turbo:update(dt)

	if self.isMotorStarted then
		local rotX, rotY, rotZ = getRotation(self.exhaustFlap);
		local rotX = Utils.degToRad((self.motor.nonClampedMotorRpm+200)/28);
		if self.motor.minRpm < 0 then
			rotX = rotX-Utils.degToRad(self.motor.minRpm/28);
		else
			rotX = rotX
		end;
		if rotX < Utils.degToRad(0) then
			rotX = Utils.degToRad(0);
		end;
		if rotX > Utils.degToRad(70) then
			rotX = Utils.degToRad(70);
		end;
		setRotation(self.exhaustFlap, rotX*1, rotY, rotZ);
	else
		setRotation(self.exhaustFlap, 0, rotY, rotZ);
	end;

	if self.isEntered then
		if InputBinding.hasEvent(InputBinding.Zetor7340TurboWORKLIGHTS) then
			self.worklightsActive = not self.worklightsActive;
		end;
		if InputBinding.hasEvent(InputBinding.HELPPANELZetor7340Turbo) then
			self.helpPanelActive = not self.helpPanelActive;
		end;
	end;
	
	for k, worklight in pairs(self.worklights) do
		setVisibility(worklight, self.worklightsActive);
	end;
		
	
	self.input = InputBinding.getAnalogInputAxis(InputBinding.AXIS_FORWARD);
	if InputBinding.isAxisZero(self.input) then
		self.input = InputBinding.getDigitalInputAxis(InputBinding.AXIS_FORWARD);
	end;
	
	local maxRpm = self.motor:getMaxRpm();
	local torque = self.motor.torqueCurve:get(self.motor.lastMotorRpm);
	if self.input ~= 0 or self.motor.speedLevel ~= 0 then
		local differenz = maxRpm-self.motor.lastMotorRpm;
		if differenz <= 510 then
			self.motor.minRpm = self.motor.minRpm+(differenz/50);
		else
			self.motor.minRpm = self.motor.minRpm-(differenz/40);
		end;
		if self.motor.minRpm <= -650 then
			self.motor.minRpm = -650;
		elseif self.motor.minRpm >= 1100 then
			self.motor.minRpm = 1100;
		end;
		if self.fieldMode then
			self.motor.minRpm = (self.motor.minRpm-120);
		else
			self.motor.minRpm = self.motor.minRpm;
		end;
	end;
	
	if self.motor.lastMotorRpm > 0 then
		self.isStarted = true;
	else
		self.isStarted = false;
	end;
	
	if self.tempOnBord then
		if self.isStarted then
			local maxRpm = self.motor:getMaxRpm();
			local powerDifferenz = ((100/self.motor.lastMotorRpm)*maxRpm)-100;
			local lastMotorRpm = self.motor.lastMotorRpm;
			if powerDifferenz > 100 then
				powerDifferenz = 100;
			elseif powerDifferenz < 0 then
				powerDifferenz = 0;
			end;
			if maxRpm < self.motor.minRpm then
				maxRpm = self.motor.minRpm;
			end;
			if self.input == 0 or self.motor.speedLevel == 0 then
				if self.temp < 80 then
					self.temp = self.temp+(4/lastMotorRpm);
				else
					self.temp = self.temp-(2/lastMotorRpm);
				end;
			end;
			if powerDifferenz > 0 then
				self.temp = self.temp+(powerDifferenz/6000);
			end;
			if self.temp > 87 then
				self.coolingOn = true;
			end;
			if self.coolingOn == true and self.temp < 81 then
				self.coolingOn = false;
			end;
			if self.coolingOn == true then
				self.temp = self.temp-(6/lastMotorRpm);
			end;
		else
			self.temp = self.temp-(self.warmUp/10);
		end;
		if self.temp < 24 then
			self.temp = 24;
		end;
	end;
    -- Reverse lights
    if self.movingDirection < 0 then
        self.reverseLightActive = true;
    else
        self.reverseLightActive = false;
    end;
    setVisibility(self.reverseLightObject, self.reverseLightActive);
        
    -- Brake lights
    setVisibility(self.brakeLightObject, self.brakeLightActive);
    
    if (self.movingDirection*self.lastSpeed*(-self.input)) < -0.001 then
        self.brakeLightActive = true;
    else
        self.brakeLightActive = false;
    end;
    
	-- Turn lights
    if self.flashLeft or self.flashRight then
        self.timer = self.timer - dt;
        if self.flashRight or self.flashWarn then
            if self.timer < 0 then
                self.rightTurnActive = not self.rightTurnActive;
            end;
        else
            self.rightTurnActive = false;
        end;
    
        if self.flashLeft or self.flashWarn then
            if self.timer < 0 then
                self.leftTurnActive = not self.leftTurnActive;
            end;
        else
            self.leftTurnActive = false;
        end;
        if self.timer < 0 then
            self.timer = 500;
        end;
    else
        self.rightTurnActive = false;
        self.leftTurnActive = false;
    end;
    setVisibility(self.rightTurnObject, self.rightTurnActive);
    setVisibility(self.leftTurnObject, self.leftTurnActive);
end;

function Zetor7340Turbo:onLeave()
	for k, worklight in pairs(self.worklights) do
		setVisibility(worklight, false);
	end;
	self.worklightsActive = false;
	self.motor.minRpm = self.backupMinRpm;
end;

function Zetor7340Turbo:draw()

	if self.helpPanelActive then
		self.helpPanelOverlay:render();
		renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.141, 0.025, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.HELPPANELZetor7340Turbo), g_i18n:getText("Zetor7340TurboHelpPanelOn")));
		renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.255, 0.024, string.format("Taste Num 1/2/3: Smerovky"));

		if not self.isMotorStarted and not self.automaticStart then	
			renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.165, 0.024, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboIGNITION), g_i18n:getText("Zetor7340TurboStart")));
		else
			renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.349, 0.024, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboIGNITION), g_i18n:getText("Zetor7340TurboStop")));
		end;
		if self.worklightsActive then
			renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.231, 0.024, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboWORKLIGHTS), g_i18n:getText("Zetor7340TurboworklightOff")));
		else
			renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.231, 0.024, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboWORKLIGHTS), g_i18n:getText("Zetor7340TurboworklightOn")));
		end;
		if self.isMotorStarted then	
			if self.motor.speedLevel ~= 0 then
				renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.165, 0.024, string.format("Taste %s/%s: %d RPM", InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboACCELERATE), InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboDECELERATE), self.motor.maxRpm[self.motor.speedLevel]));
			else
				renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.165, 0.024, string.format("Taste %s/%s: %d RPM", InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboACCELERATE), InputBinding.getButtonKeyName(InputBinding.Zetor7340TurboDECELERATE), self.motor.maxRpm[3]));
			end;
			if self.reverse then
				renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.189, 0.024, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.Zetor7340Turbo_REVERSE), g_i18n:getText("Zetor7340TurboshuttleReverser1")));
			else
				renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.189, 0.024, string.format("Taste %s: %s", InputBinding.getButtonKeyName(InputBinding.Zetor7340Turbo_REVERSE), g_i18n:getText("Zetor7340TurboshuttleReverser2")));
			end;
		if self.tempOnBord then
			local red = 0.0;
			local green = 0.0;
			local blue = 0.0;
			if self.temp < 63 then
				red = 0.0;
				green = 0.0;
				blue = 1.0;
			elseif self.temp >= 63 and self.temp <= 92 then
				blue = 0.0;
				red = 0.0;
				green = 1.0;
			elseif self.temp > 92 then
				red = 1.0;
				green = 0.0;
				blue = 0.0;
			end;
			setTextColor(red, green, blue, 1.0);
			renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.207, 0.024, string.format("                          %d 'C", self.temp));
			setTextColor(1.0, 1.0, 1.0, 1.0);
			renderText(self.helpPanelTextPosX, self.helpPanelTextPosY-0.207, 0.024, string.format("Teplota motoru:"));
		end;
    end;
	else
		if not self.isMotorStarted and not self.automaticStart then	
			g_currentMission:addHelpButtonText(g_i18n:getText("Zetor7340TurboStart"), InputBinding.Zetor7340TurboIGNITION);
		end;
		g_currentMission:addHelpButtonText(g_i18n:getText("Zetor7340TurboHelpPanelOff"), InputBinding.HELPPANELZetor7340Turbo);
	    end;

end;

function Zetor7340Turbo:validateAttacherJoint(implement, jointDesc, dt)
    return true;
end;

function Zetor7340Turbo:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

	local loadTemp = getXMLFloat(xmlFile, key.."#temp");
	if loadTemp ~= nil then
		self.temp = loadTemp;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function Zetor7340Turbo:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'temp="'..self.temp..'"';
    return attributes, nil;
end;